from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CommonCDN
from cdnmon.util.cidr import deduplicate_networks


class GoogleCIDR(CIDR):
    ipv4_url: str = "https://www.gstatic.com/ipranges/cloud.json"
    ipv6_url: str = ipv4_url

    def extract(self):
        """
        [1] https://support.google.com/a/answer/10026322
        [2] https://www.gstatic.com/ipranges/goog.json
        [3] https://www.gstatic.com/ipranges/cloud.json
        """
        import ipaddress

        data = self.http_get(self.ipv4_url).json()

        ipv4_networks = []
        ipv6_networks = []

        prefixes = data["prefixes"]
        for prefix in prefixes:
            if "ipv4Prefix" in prefix:
                ipv4_networks.append(str(ipaddress.IPv4Network(prefix.get("ipv4Prefix"), strict=False)))

            if "ipv6Prefix" in prefix:
                ipv6_networks.append(str(ipaddress.IPv6Network(prefix.get("ipv6Prefix"), strict=False)))

        return {
            "ipv4": "\n".join(ipv4_networks),
            "ipv6": "\n".join(ipv6_networks),
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="google",
    asn_patterns=["google"],
    cname_suffixes=[],
    cidr=GoogleCIDR(),
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.util.cidr import deduplicate_networks


class AkamaiCIDR(CIDR):
    ipv4_url: str = "https://techdocs.akamai.com/property-manager/pdfs/akamai_ipv4_ipv6_CIDRs-txt.zip"
    ipv6_url: str = ipv4_url

    def extract(self):
        """
        [1] https://techdocs.akamai.com/property-mgr/docs/origin-ip-access-control
        """
        import zipfile
        import io

        content = self.http_get(AkamaiCIDR.ipv4_url).content
        z = zipfile.ZipFile(io.BytesIO(content), mode="r")
        return {
            "ipv4": z.read("akamai_ipv4_CIDRs.txt").decode("utf-8"),
            "ipv6": z.read("akamai_ipv6_CIDRs.txt").decode("utf-8"),
        }

    def transform(self, response: dict):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="akamai",
    asn_patterns=["akamai"],
    cname_suffixes=[
        CNAMEPattern(suffix=".edgesuite.net", pattern=r"${domain}.edgesuite.net"),
        CNAMEPattern(suffix=".edgekey.net", pattern=r"${domain}.edgekey.net"),
        CNAMEPattern(suffix="-v1.akamaized.net", pattern=r"${domain}-v1.akamaized.net"),
        CNAMEPattern(suffix="-v1.edgekey.net", pattern=r"${domain}-v1.edgekey.net"),
        CNAMEPattern(suffix=".akadns.net"),
        CNAMEPattern(suffix=".akamai.net"),
    ],
    cidr=AkamaiCIDR(),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="alibaba-cdn",
    asn_patterns=["alibaba", "taobao", "alicloud"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdngslb.com", pattern=r"${domain}.cdngslb.com"),
        CNAMEPattern(suffix=".queniuaa.com", pattern=r"${domain}.queniuaa.com"),
        CNAMEPattern(suffix=".queniuab.com", pattern=r"${domain}.queniuab.com"),
        CNAMEPattern(suffix=".queniuai.com", pattern=r"${domain}.queniuai.com"),
        CNAMEPattern(suffix=".queniuak.com", pattern=r"${domain}.queniuak.com"),
        CNAMEPattern(suffix=".queniual.com", pattern=r"${domain}.queniual.com"),
        CNAMEPattern(suffix=".queniuam.com", pattern=r"${domain}.queniuam.com"),
        CNAMEPattern(suffix=".queniubd.com", pattern=r"${domain}.queniubd.com"),
        CNAMEPattern(suffix=".queniubg.com", pattern=r"${domain}.queniubg.com"),
        CNAMEPattern(suffix=".queniubh.com", pattern=r"${domain}.queniubh.com"),
        CNAMEPattern(suffix=".queniubk.com", pattern=r"${domain}.queniubk.com"),
        CNAMEPattern(suffix=".queniubl.com", pattern=r"${domain}.queniubl.com"),
        CNAMEPattern(suffix=".queniubm.com", pattern=r"${domain}.queniubm.com"),
        CNAMEPattern(suffix=".queniubn.com", pattern=r"${domain}.queniubn.com"),
        CNAMEPattern(suffix=".queniucdn.com", pattern=r"${domain}.queniucdn.com"),
        CNAMEPattern(suffix=".queniucf.com", pattern=r"${domain}.queniucf.com"),
        CNAMEPattern(suffix=".queniucg.com", pattern=r"${domain}.queniucg.com"),
        CNAMEPattern(suffix=".queniuck.com", pattern=r"${domain}.queniuck.com"),
        CNAMEPattern(suffix=".queniucl.com", pattern=r"${domain}.queniucl.com"),
        CNAMEPattern(suffix=".queniucm.com", pattern=r"${domain}.queniucm.com"),
        CNAMEPattern(suffix=".queniucn.com", pattern=r"${domain}.queniucn.com"),
        CNAMEPattern(suffix=".queniuco.com", pattern=r"${domain}.queniuco.com"),
        CNAMEPattern(suffix=".queniucs.com", pattern=r"${domain}.queniucs.com"),
        CNAMEPattern(suffix=".queniucx.com", pattern=r"${domain}.queniucx.com"),
        CNAMEPattern(suffix=".queniucz.com", pattern=r"${domain}.queniucz.com"),
        CNAMEPattern(suffix=".queniuda.com", pattern=r"${domain}.queniuda.com"),
        CNAMEPattern(suffix=".queniudd.com", pattern=r"${domain}.queniudd.com"),
        CNAMEPattern(suffix=".queniudg.com", pattern=r"${domain}.queniudg.com"),
        CNAMEPattern(suffix=".queniudl.com", pattern=r"${domain}.queniudl.com"),
        CNAMEPattern(suffix=".queniudns.com", pattern=r"${domain}.queniudns.com"),
        CNAMEPattern(suffix=".queniudo.com", pattern=r"${domain}.queniudo.com"),
        CNAMEPattern(suffix=".queniudt.com", pattern=r"${domain}.queniudt.com"),
        CNAMEPattern(suffix=".queniuee.com", pattern=r"${domain}.queniuee.com"),
        CNAMEPattern(suffix=".queniueg.com", pattern=r"${domain}.queniueg.com"),
        CNAMEPattern(suffix=".queniueh.com", pattern=r"${domain}.queniueh.com"),
        CNAMEPattern(suffix=".queniuei.com", pattern=r"${domain}.queniuei.com"),
        CNAMEPattern(suffix=".queniues.com", pattern=r"${domain}.queniues.com"),
        CNAMEPattern(suffix=".queniuez.com", pattern=r"${domain}.queniuez.com"),
        CNAMEPattern(suffix=".queniufa.com", pattern=r"${domain}.queniufa.com"),
        CNAMEPattern(suffix=".queniufm.com", pattern=r"${domain}.queniufm.com"),
        CNAMEPattern(suffix=".queniufr.com", pattern=r"${domain}.queniufr.com"),
        CNAMEPattern(suffix=".queniufw.com", pattern=r"${domain}.queniufw.com"),
        CNAMEPattern(suffix=".queniufz.com", pattern=r"${domain}.queniufz.com"),
        CNAMEPattern(suffix=".queniugd.com", pattern=r"${domain}.queniugd.com"),
        CNAMEPattern(suffix=".queniugm.com", pattern=r"${domain}.queniugm.com"),
        CNAMEPattern(suffix=".queniugslb.com", pattern=r"${domain}.queniugslb.com"),
        CNAMEPattern(suffix=".queniugv.com", pattern=r"${domain}.queniugv.com"),
        CNAMEPattern(suffix=".queniugz.com", pattern=r"${domain}.queniugz.com"),
        CNAMEPattern(suffix=".queniuhg.com", pattern=r"${domain}.queniuhg.com"),
        CNAMEPattern(suffix=".queniuhl.com", pattern=r"${domain}.queniuhl.com"),
        CNAMEPattern(suffix=".queniuhx.com", pattern=r"${domain}.queniuhx.com"),
        CNAMEPattern(suffix=".queniuhy.com", pattern=r"${domain}.queniuhy.com"),
        CNAMEPattern(suffix=".queniuhz.com", pattern=r"${domain}.queniuhz.com"),
        CNAMEPattern(suffix=".queniuic.com", pattern=r"${domain}.queniuic.com"),
        CNAMEPattern(suffix=".queniuif.com", pattern=r"${domain}.queniuif.com"),
        CNAMEPattern(suffix=".queniuin.com", pattern=r"${domain}.queniuin.com"),
        CNAMEPattern(suffix=".queniuiq.com", pattern=r"${domain}.queniuiq.com"),
        CNAMEPattern(suffix=".queniuit.com", pattern=r"${domain}.queniuit.com"),
        CNAMEPattern(suffix=".queniuiy.com", pattern=r"${domain}.queniuiy.com"),
        CNAMEPattern(suffix=".queniuja.com", pattern=r"${domain}.queniuja.com"),
        CNAMEPattern(suffix=".queniujd.com", pattern=r"${domain}.queniujd.com"),
        CNAMEPattern(suffix=".queniujf.com", pattern=r"${domain}.queniujf.com"),
        CNAMEPattern(suffix=".queniujg.com", pattern=r"${domain}.queniujg.com"),
        CNAMEPattern(suffix=".queniujq.com", pattern=r"${domain}.queniujq.com"),
        CNAMEPattern(suffix=".queniujz.com", pattern=r"${domain}.queniujz.com"),
        CNAMEPattern(suffix=".queniukk.com", pattern=r"${domain}.queniukk.com"),
        CNAMEPattern(suffix=".queniukq.com", pattern=r"${domain}.queniukq.com"),
        CNAMEPattern(suffix=".queniukr.com", pattern=r"${domain}.queniukr.com"),
        CNAMEPattern(suffix=".queniuks.com", pattern=r"${domain}.queniuks.com"),
        CNAMEPattern(suffix=".queniukt.com", pattern=r"${domain}.queniukt.com"),
        CNAMEPattern(suffix=".queniukw.com", pattern=r"${domain}.queniukw.com"),
        CNAMEPattern(suffix=".queniukx.com", pattern=r"${domain}.queniukx.com"),
        CNAMEPattern(suffix=".queniulf.com", pattern=r"${domain}.queniulf.com"),
        CNAMEPattern(suffix=".queniulg.com", pattern=r"${domain}.queniulg.com"),
        CNAMEPattern(suffix=".queniuli.com", pattern=r"${domain}.queniuli.com"),
        CNAMEPattern(suffix=".queniulp.com", pattern=r"${domain}.queniulp.com"),
        CNAMEPattern(suffix=".queniulr.com", pattern=r"${domain}.queniulr.com"),
        CNAMEPattern(suffix=".queniuls.com", pattern=r"${domain}.queniuls.com"),
        CNAMEPattern(suffix=".queniulv.com", pattern=r"${domain}.queniulv.com"),
        CNAMEPattern(suffix=".queniumd.com", pattern=r"${domain}.queniumd.com"),
        CNAMEPattern(suffix=".queniumf.com", pattern=r"${domain}.queniumf.com"),
        CNAMEPattern(suffix=".queniumn.com", pattern=r"${domain}.queniumn.com"),
        CNAMEPattern(suffix=".queniumo.com", pattern=r"${domain}.queniumo.com"),
        CNAMEPattern(suffix=".queniumy.com", pattern=r"${domain}.queniumy.com"),
        CNAMEPattern(suffix=".queniunh.com", pattern=r"${domain}.queniunh.com"),
        CNAMEPattern(suffix=".queniuni.com", pattern=r"${domain}.queniuni.com"),
        CNAMEPattern(suffix=".queniunq.com", pattern=r"${domain}.queniunq.com"),
        CNAMEPattern(suffix=".queniuns.com", pattern=r"${domain}.queniuns.com"),
        CNAMEPattern(suffix=".queniuny.com", pattern=r"${domain}.queniuny.com"),
        CNAMEPattern(suffix=".queniunz.com", pattern=r"${domain}.queniunz.com"),
        CNAMEPattern(suffix=".queniuod.com", pattern=r"${domain}.queniuod.com"),
        CNAMEPattern(suffix=".queniuog.com", pattern=r"${domain}.queniuog.com"),
        CNAMEPattern(suffix=".queniuop.com", pattern=r"${domain}.queniuop.com"),
        CNAMEPattern(suffix=".queniuos.com", pattern=r"${domain}.queniuos.com"),
        CNAMEPattern(suffix=".queniuot.com", pattern=r"${domain}.queniuot.com"),
        CNAMEPattern(suffix=".queniuoy.com", pattern=r"${domain}.queniuoy.com"),
        CNAMEPattern(suffix=".queniupc.com", pattern=r"${domain}.queniupc.com"),
        CNAMEPattern(suffix=".queniupe.com", pattern=r"${domain}.queniupe.com"),
        CNAMEPattern(suffix=".queniupg.com", pattern=r"${domain}.queniupg.com"),
        CNAMEPattern(suffix=".queniupj.com", pattern=r"${domain}.queniupj.com"),
        CNAMEPattern(suffix=".queniupl.com", pattern=r"${domain}.queniupl.com"),
        CNAMEPattern(suffix=".queniupq.com", pattern=r"${domain}.queniupq.com"),
        CNAMEPattern(suffix=".queniupz.com", pattern=r"${domain}.queniupz.com"),
        CNAMEPattern(suffix=".queniuqe.com", pattern=r"${domain}.queniuqe.com"),
        CNAMEPattern(suffix=".queniuql.com", pattern=r"${domain}.queniuql.com"),
        CNAMEPattern(suffix=".queniuqm.com", pattern=r"${domain}.queniuqm.com"),
        CNAMEPattern(suffix=".queniuqt.com", pattern=r"${domain}.queniuqt.com"),
        CNAMEPattern(suffix=".queniuqy.com", pattern=r"${domain}.queniuqy.com"),
        CNAMEPattern(suffix=".queniuqz.com", pattern=r"${domain}.queniuqz.com"),
        CNAMEPattern(suffix=".queniurc.com", pattern=r"${domain}.queniurc.com"),
        CNAMEPattern(suffix=".queniurj.com", pattern=r"${domain}.queniurj.com"),
        CNAMEPattern(suffix=".queniurk.com", pattern=r"${domain}.queniurk.com"),
        CNAMEPattern(suffix=".queniurq.com", pattern=r"${domain}.queniurq.com"),
        CNAMEPattern(suffix=".queniuru.com", pattern=r"${domain}.queniuru.com"),
        CNAMEPattern(suffix=".queniurv.com", pattern=r"${domain}.queniurv.com"),
        CNAMEPattern(suffix=".queniurx.com", pattern=r"${domain}.queniurx.com"),
        CNAMEPattern(suffix=".queniusa.com", pattern=r"${domain}.queniusa.com"),
        CNAMEPattern(suffix=".queniusf.com", pattern=r"${domain}.queniusf.com"),
        CNAMEPattern(suffix=".queniusg.com", pattern=r"${domain}.queniusg.com"),
        CNAMEPattern(suffix=".queniusi.com", pattern=r"${domain}.queniusi.com"),
        CNAMEPattern(suffix=".queniuso.com", pattern=r"${domain}.queniuso.com"),
        CNAMEPattern(suffix=".queniusp.com", pattern=r"${domain}.queniusp.com"),
        CNAMEPattern(suffix=".queniusr.com", pattern=r"${domain}.queniusr.com"),
        CNAMEPattern(suffix=".queniusy.com", pattern=r"${domain}.queniusy.com"),
        CNAMEPattern(suffix=".queniusz.com", pattern=r"${domain}.queniusz.com"),
        CNAMEPattern(suffix=".queniutc.com", pattern=r"${domain}.queniutc.com"),
        CNAMEPattern(suffix=".queniutn.com", pattern=r"${domain}.queniutn.com"),
        CNAMEPattern(suffix=".queniutt.com", pattern=r"${domain}.queniutt.com"),
        CNAMEPattern(suffix=".queniutz.com", pattern=r"${domain}.queniutz.com"),
        CNAMEPattern(suffix=".queniuua.com", pattern=r"${domain}.queniuua.com"),
        CNAMEPattern(suffix=".queniuuc.com", pattern=r"${domain}.queniuuc.com"),
        CNAMEPattern(suffix=".queniuuf.com", pattern=r"${domain}.queniuuf.com"),
        CNAMEPattern(suffix=".queniuuk.com", pattern=r"${domain}.queniuuk.com"),
        CNAMEPattern(suffix=".queniuum.com", pattern=r"${domain}.queniuum.com"),
        CNAMEPattern(suffix=".queniuuq.com", pattern=r"${domain}.queniuuq.com"),
        CNAMEPattern(suffix=".queniuus.com", pattern=r"${domain}.queniuus.com"),
        CNAMEPattern(suffix=".queniuuy.com", pattern=r"${domain}.queniuuy.com"),
        CNAMEPattern(suffix=".queniuvb.com", pattern=r"${domain}.queniuvb.com"),
        CNAMEPattern(suffix=".queniuvc.com", pattern=r"${domain}.queniuvc.com"),
        CNAMEPattern(suffix=".queniuvi.com", pattern=r"${domain}.queniuvi.com"),
        CNAMEPattern(suffix=".queniuvk.com", pattern=r"${domain}.queniuvk.com"),
        CNAMEPattern(suffix=".queniuvq.com", pattern=r"${domain}.queniuvq.com"),
        CNAMEPattern(suffix=".queniuvu.com", pattern=r"${domain}.queniuvu.com"),
        CNAMEPattern(suffix=".queniuvx.com", pattern=r"${domain}.queniuvx.com"),
        CNAMEPattern(suffix=".queniuwb.com", pattern=r"${domain}.queniuwb.com"),
        CNAMEPattern(suffix=".queniuwh.com", pattern=r"${domain}.queniuwh.com"),
        CNAMEPattern(suffix=".queniuwk.com", pattern=r"${domain}.queniuwk.com"),
        CNAMEPattern(suffix=".queniuwx.com", pattern=r"${domain}.queniuwx.com"),
        CNAMEPattern(suffix=".queniuxk.com", pattern=r"${domain}.queniuxk.com"),
        CNAMEPattern(suffix=".queniuxr.com", pattern=r"${domain}.queniuxr.com"),
        CNAMEPattern(suffix=".queniuxx.com", pattern=r"${domain}.queniuxx.com"),
        CNAMEPattern(suffix=".queniuyb.com", pattern=r"${domain}.queniuyb.com"),
        CNAMEPattern(suffix=".queniuyg.com", pattern=r"${domain}.queniuyg.com"),
        CNAMEPattern(suffix=".queniuyk.com", pattern=r"${domain}.queniuyk.com"),
        CNAMEPattern(suffix=".queniuys.com", pattern=r"${domain}.queniuys.com"),
        CNAMEPattern(suffix=".queniuyu.com", pattern=r"${domain}.queniuyu.com"),
        CNAMEPattern(suffix=".queniuyz.com", pattern=r"${domain}.queniuyz.com"),
        CNAMEPattern(suffix=".queniuza.com", pattern=r"${domain}.queniuza.com"),
        CNAMEPattern(suffix=".queniuzd.com", pattern=r"${domain}.queniuzd.com"),
        CNAMEPattern(suffix=".queniuzf.com", pattern=r"${domain}.queniuzf.com"),
        CNAMEPattern(suffix=".queniuzl.com", pattern=r"${domain}.queniuzl.com"),
        CNAMEPattern(suffix=".queniuzp.com", pattern=r"${domain}.queniuzp.com"),
        CNAMEPattern(suffix=".queniuzv.com", pattern=r"${domain}.queniuzv.com"),
        CNAMEPattern(suffix=".queniuzy.com", pattern=r"${domain}.queniuzy.com"),
        CNAMEPattern(suffix=".queniuzz.com", pattern=r"${domain}.queniuzz.com"),
    ],
    cidr=BGPViewCIDR(["alibaba", "taobao", "alicloud"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="chinanet",
    asn_patterns=["chinanet"],
    cname_suffixes=[
        CNAMEPattern(suffix=".iname.damddos.com", pattern=r"${domain}.iname.damddos.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["chinanet"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="alibaba-dcdn",
    asn_patterns=["alibaba", "taobao", "alicloud"],
    cname_suffixes=[
        CNAMEPattern(suffix=".m.alikunlun.com", pattern=r"${domain}.m.alikunlun.com"),
        CNAMEPattern(suffix=".w.kunlunaq.com", pattern=r"${domain}.w.kunlunaq.com"),
        CNAMEPattern(suffix=".w.kunlunar.com", pattern=r"${domain}.w.kunlunar.com"),
        CNAMEPattern(suffix=".w.kunlunca.com", pattern=r"${domain}.w.kunlunca.com"),
        CNAMEPattern(suffix=".w.kunluncan.com", pattern=r"${domain}.w.kunluncan.com"),
        CNAMEPattern(suffix=".w.kunlunea.com", pattern=r"${domain}.w.kunlunea.com"),
        CNAMEPattern(suffix=".w.kunlungem.com", pattern=r"${domain}.w.kunlungem.com"),
        CNAMEPattern(suffix=".w.kunlungr.com", pattern=r"${domain}.w.kunlungr.com"),
        CNAMEPattern(suffix=".w.kunlunhuf.com", pattern=r"${domain}.w.kunlunhuf.com"),
        CNAMEPattern(suffix=".w.kunlunle.com", pattern=r"${domain}.w.kunlunle.com"),
        CNAMEPattern(suffix=".w.kunlunno.com", pattern=r"${domain}.w.kunlunno.com"),
        CNAMEPattern(suffix=".w.kunlunpi.com", pattern=r"${domain}.w.kunlunpi.com"),
        CNAMEPattern(suffix=".w.kunlunsl.com", pattern=r"${domain}.w.kunlunsl.com"),
        CNAMEPattern(suffix=".w.kunlunso.com", pattern=r"${domain}.w.kunlunso.com"),
    ],
    cidr=BGPViewCIDR(["alibaba", "taobao", "alicloud"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="edgecast",
    asn_patterns=["edgecast"],
    cname_suffixes=[
        CNAMEPattern(suffix=".edgecastcdn.net"),
    ],
    cidr=BGPViewCIDR(["edgecast"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="baidu",
    asn_patterns=["baidu"],
    cname_suffixes=[
        CNAMEPattern(suffix=".a.bdydns.com", pattern=r"${domain}.a.bdydns.com"),
        CNAMEPattern(suffix=".yjs-cdn.com", pattern=r"${domain}.yjs-cdn.com"),
        CNAMEPattern(suffix=".yunjiasu-cdn.net", pattern=r"${domain}.yunjiasu-cdn.net"),
    ],
    cidr=BGPViewCIDR(["baidu"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="asia-isp",
    asn_patterns=["asia-isp"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cy-isp.com", pattern=r"${domain}.cy-isp.com"),
    ],
    cidr=BGPViewCIDR(["asia-isp"]),
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.util.cidr import deduplicate_networks


class MaxCDNCIDR(CIDR):
    ipv4_url: str = "https://support.maxcdn.com/hc/en-us/article_attachments/360051920551/maxcdn_ips.txt"
    ipv6_url: str = ipv4_url

    def extract(self):
        """
        [1] https://support.maxcdn.com/one/tutorial/ip-blocks/
        [2] https://support.maxcdn.com/one/assets/ips.txt (302 Redirected to [3])
        [3] https://support.maxcdn.com/hc/en-us/article_attachments/360051920551/maxcdn_ips.txt
        """
        text = self.http_get(self.ipv4_url).text
        return {
            "ipv4": text,
            "ipv6": text,
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="maxcdn",
    asn_patterns=["maxcdn"],
    cname_suffixes=[
        CNAMEPattern(suffix=".netdna-cdn.com"),
        CNAMEPattern(suffix=".netdna-ssl.com"),
        CNAMEPattern(suffix=".netdna.com"),
    ],
    cidr=MaxCDNCIDR(),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="edgenext",
    asn_patterns=["edgenext"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(["edgenext"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="ucloud",
    asn_patterns=["ucloud"],
    cname_suffixes=[
        CNAMEPattern(suffix=".ucloud.com.cn", pattern=r"${domain}.ucloud.com.cn"),
        CNAMEPattern(suffix=".ucloudnaming.cn", pattern=r"${domain}.ucloudnaming.cn"),
        CNAMEPattern(suffix=".ucloudnaming.info", pattern=r"${domain}.ucloudnaming.info"),
        CNAMEPattern(suffix=".ugslb.net", pattern=r"${domain}.ugslb.net"),
    ],
    cidr=BGPViewCIDR(query_term_list=["ucloud"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="douyin",
    asn_patterns=["bytedance", "douyin"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdnbuild.com", pattern=r"${domain}.cdnbuild.com"),
    ],
    cidr=BGPViewCIDR(["bytedance", "douyin"]),
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.util.cidr import deduplicate_networks


class CloudflareCIDR(CIDR):
    ipv4_url: str = "https://www.cloudflare.com/ips-v4"
    ipv6_url: str = "https://www.cloudflare.com/ips-v6"

    def extract(self):
        return {
            "ipv4": self.http_get(self.ipv4_url).text,
            "ipv6": self.http_get(self.ipv6_url).text,
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="cloudflare",
    asn_patterns=["cloudflare"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdn.cloudflare.net", pattern=r"${domain}.cdn.cloudflare.net"),
    ],
    cidr=CloudflareCIDR(),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="volc",
    asn_patterns=["bytedance"],
    cname_suffixes=[
        CNAMEPattern(suffix=".volcgslb.com", pattern=r"${domain}.volcgslb.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["bytedance"]),
)



import os

from pymongo import MongoClient


def get_mongo_collection(database, collection):
    # e.g. MONGODB_URI=mongodb://mongo:mongo@127.0.0.1:27017/
    client = MongoClient(os.getenv("MONGODB_URI"))
    db = client[database]
    return db[collection]



import ipaddress

import requests


def get_ip_range_of_aws(pattern):
    data = requests.get("https://ip-ranges.amazonaws.com/ip-ranges.json").json()

    ipv4_networks = []
    ipv4_prefixes = data["prefixes"]
    for ipv4_prefix in ipv4_prefixes:
        if ipv4_prefix["service"] == pattern:
            ipv4_networks.append(str(ipaddress.IPv4Network(ipv4_prefix["ip_prefix"], strict=False)))

    ipv6_networks = []
    ipv6_prefixes = data["ipv6_prefixes"]
    for ipv6_prefix in ipv6_prefixes:
        if ipv6_prefix["service"] == pattern:
            ipv6_networks.append(str(ipaddress.IPv6Network(ipv6_prefix["ipv6_prefix"], strict=False)))

    return ipv4_networks, ipv6_networks



import glob
import importlib
import os

from cdnmon.model.cdn import CommonCDN


def CDN(name: str) -> CommonCDN:
    module_name = f"cdnmon.model.cdn.{name.replace('-', '_')}"
    module = importlib.import_module(module_name)
    return module.CDN


__all__ = ["CDN"]


def main():
    for path in glob.glob(os.path.join(os.path.dirname(os.path.dirname(__file__)), "data", "*.yaml")):
        cdn_name, _ = os.path.splitext(os.path.basename(path))
        cdn = CDN(cdn_name)
        print(cdn)


if __name__ == "__main__":
    main()



import functools
import sys

import humanize
import requests
from loguru import logger

logger.remove()
logger.add(sys.stderr, level="INFO")


class BGPViewClient:
    @staticmethod
    @functools.lru_cache(maxsize=None)
    def search(query_term: str):
        """
        [1] https://bgpview.docs.apiary.io/#reference/0/asn-prefixes/view-asn-prefixes
        [2] https://api.bgpview.io/search
        [3] https://api.bgpview.io/search?query_term=tencent
        """
        url = f"https://bgpview.execve.workers.dev/search?query_term={query_term}"
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36"
        }
        response = requests.get(url, headers=headers)
        logger.info(
            f"{response.status_code} {response.reason} | GET {url}  ({humanize.naturalsize(len(response.content))} Bytes)"
        )
        return response.json()


def main():
    BGPViewClient.transform(BGPViewClient.search("alicloud"))


if __name__ == "__main__":
    main()



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="qihoo",
    asn_patterns=["qihoo"],
    cname_suffixes=[
        CNAMEPattern(suffix=".360qhcdn.com", pattern=r"${domain}.360qhcdn.com"),
        CNAMEPattern(suffix=".qhcdn-lb.com", pattern=r"${domain}.qhcdn-lb.com"),
        CNAMEPattern(suffix=".qhcdn.com", pattern=r"${domain}.qhcdn.com"),
        CNAMEPattern(suffix=".360imgcdn.com", pattern=r"${domain}.360imgcdn.com"),
        CNAMEPattern(suffix=".qhdlcdn.com", pattern=r"${domain}.qhdlcdn.com"),
        CNAMEPattern(suffix=".qh-cdn.com", pattern=r"${domain}.qh-cdn.com"),
        CNAMEPattern(suffix=".360dlcdn.com", pattern=r"${domain}.360dlcdn.com"),
        CNAMEPattern(suffix=".360tpcdn.com", pattern=r"${domain}.360tpcdn.com"),
    ],
    cidr=BGPViewCIDR(["qihoo"]),
)



from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.model.cdn.fastly import FastlyCIDR

CDN = CommonCDN(
    name="fastly-edge-compute",
    asn_patterns=["fastly"],
    cname_suffixes=[
        CNAMEPattern(suffix=".edgecompute.app"),
    ],
    cidr=FastlyCIDR(),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="jingdong",
    asn_patterns=["jingdong"],
    cname_suffixes=[
        CNAMEPattern(suffix=".dy.galileo.jcloud-cdn.com", pattern=r"${domain}.dy.galileo.jcloud-cdn.com"),
        CNAMEPattern(suffix=".cdn.jcloudcdn.com", pattern=r"${domain}.cdn.jcloudcdn.com"),
        CNAMEPattern(suffix=".jcloudcache.com", pattern=r"${domain}.lk-[0-9a-f]{6}.jcloudcache.com"),
        CNAMEPattern(suffix=".cdn.jcloudcache.com", pattern=r"${domain}.lk-[0-9a-f]{6}.cdn.jcloudcache.com"),
        CNAMEPattern(suffix=".jcloudimg.com"),
        CNAMEPattern(suffix=".gslb.qianxun.com", pattern=r"${domain}.gslb.qianxun.com"),
    ],
    cidr=BGPViewCIDR(["jingdong"]),
)



from cdnmon.model.cdn import CIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN
from cdnmon.util.aws import get_ip_range_of_aws
from cdnmon.util.cidr import deduplicate_networks


class CloudFrontCIDR(CIDR):
    ipv4_url: str = "https://ip-ranges.amazonaws.com/ip-ranges.json"
    ipv6_url: str = ipv4_url

    def extract(self):
        """
        [1] https://docs.aws.amazon.com/general/latest/gr/aws-ip-ranges.html
        """
        ipv4_networks, ipv6_networks = get_ip_range_of_aws("CLOUDFRONT")
        return {
            "ipv4": "\n".join(ipv4_networks),
            "ipv6": "\n".join(ipv6_networks),
        }

    def transform(self, response):
        ipv4_networks = [line.strip() for line in response["ipv4"].split("\n") if line.strip() != ""]
        ipv6_networks = [line.strip() for line in response["ipv6"].split("\n") if line.strip() != ""]
        return {
            "ipv4_prefixes": deduplicate_networks(ipv4_networks, filter_version=4),
            "ipv6_prefixes": deduplicate_networks(ipv6_networks, filter_version=6),
        }


CDN = CommonCDN(
    name="cloudfront",
    asn_patterns=["cloudfront", "amazon"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cloudfront.net"),
    ],
    cidr=CloudFrontCIDR(),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="advancedhosting",
    asn_patterns=["advancedhosting"],
    cname_suffixes=[
        CNAMEPattern(suffix=".ahacdn.me"),
    ],
    cidr=BGPViewCIDR(query_term_list=["advancedhosting"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="chinacache",
    asn_patterns=["chinacache"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(["chinacache"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cachefly",
    asn_patterns=["cachefly"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(["cachefly"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="ctyun",
    asn_patterns=["ctyun", "chinanet"],
    cname_suffixes=[
        CNAMEPattern(suffix=".ctadns.cn", pattern=r"${domain}.ctadns.cn"),
        CNAMEPattern(suffix=".ctacdn.cn", pattern=r"${domain}.ctacdn.cn"),
    ],
    cidr=BGPViewCIDR(["ctyun", "chinanet"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="upyun",
    asn_patterns=["youpai", "upyun"],
    cname_suffixes=[
        CNAMEPattern(suffix=".aicdn.com", pattern=r"[0-9a-f]{12}.b0.aicdn.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["youpai", "upyun"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="wangsu",
    asn_patterns=["wangsu"],
    cname_suffixes=[
        CNAMEPattern(suffix=".lxdns.com", pattern=r"${domain}.lxdns.com"),
        CNAMEPattern(suffix=".wscdns.com", pattern=r"${domain}.wscdns.com"),
        CNAMEPattern(suffix=".wscvip.cn", pattern=r"${domain}.wscvip.cn"),
        CNAMEPattern(suffix=".wscvip.com", pattern=r"${domain}.wscvip.com"),
        CNAMEPattern(suffix=".wsdvs.com", pattern=r"${domain}.wsdvs.com"),
        CNAMEPattern(suffix=".wsglb0.cn", pattern=r"${domain}.wsglb0.cn"),
        CNAMEPattern(suffix=".wsglb0.com", pattern=r"${domain}.wsglb0.com"),
        CNAMEPattern(suffix=".wsssec.com", pattern=r"${domain}.wsssec.com"),
        CNAMEPattern(suffix=".wswebpic.com", pattern=r"${domain}.wswebpic.com"),
        CNAMEPattern(suffix=".cdn20.com"),
        CNAMEPattern(suffix=".cdn30.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["wangsu"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cdnetworks",
    asn_patterns=["cdnetworks"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(["cdnetworks"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="tencent",
    asn_patterns=["tencent"],
    cname_suffixes=[
        CNAMEPattern(suffix=".dsa.dnsv1.com", pattern=r"${domain}.dsa.dnsv1.com"),
        CNAMEPattern(suffix=".dsa.dnsv1.com.cn", pattern=r"${domain}.dsa.dnsv1.com.cn"),
        CNAMEPattern(suffix=".cdn.dnsv1.com", pattern=r"${domain}.cdn.dnsv1.com"),
        CNAMEPattern(suffix=".cdn.dnsv1.com.cn", pattern=r"${domain}.cdn.dnsv1.com.cn"),
        CNAMEPattern(suffix=".eo.dnse0.com", pattern=r"${domain}.eo.dnse0.com"),
        CNAMEPattern(suffix=".eo.dnse1.com", pattern=r"${domain}.eo.dnse1.com"),
        CNAMEPattern(suffix=".eo.dnse2.com", pattern=r"${domain}.eo.dnse2.com"),
        CNAMEPattern(suffix=".eo.dnse3.com", pattern=r"${domain}.eo.dnse3.com"),
        CNAMEPattern(suffix=".eo.dnse4.com", pattern=r"${domain}.eo.dnse4.com"),
        CNAMEPattern(suffix=".eo.dnse5.com", pattern=r"${domain}.eo.dnse5.com"),
        CNAMEPattern(suffix=".cdn.qcloudcdn.cn", pattern=r"${domain}.cdn.qcloudcdn.cn"),
        CNAMEPattern(suffix=".txlivecdn.com", pattern=r"${domain}.txlivecdn.com"),
        CNAMEPattern(suffix=".ovscdns.com", pattern=r"${domain}.ovscdns.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["tencent"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="frontdoor",
    asn_patterns=["frontdoor", "azure", "microsoft"],
    cname_suffixes=[
        CNAMEPattern(suffix=".azurefd.net", pattern=r"${name}.azurefd.net"),
    ],
    cidr=BGPViewCIDR(["frontdoor", "azure", "microsoft"]),
)



import datetime
import functools
import ipaddress
import os
import sys
import tempfile
from dataclasses import dataclass
from dataclasses import field
from typing import List

import humanize
import requests
import yaml
from loguru import logger

from cdnmon.util import bgpview
from cdnmon.util import db
from cdnmon.util.cidr import deduplicate_networks

logger.remove()
logger.add(sys.stderr, level="INFO")


class ETL:
    def extract(self):
        raise NotImplementedError

    def transform(self, data):
        raise NotImplementedError

    def load(self, data):
        raise NotImplementedError


class CIDR(ETL):
    def ipv4_prefixes(self) -> List[str]:
        return self.transform(self.extract())["ipv4_prefixes"]

    def ipv6_prefixes(self) -> List[str]:
        return self.transform(self.extract())["ipv6_prefixes"]

    @functools.lru_cache(maxsize=None)
    def http_get(self, url):
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36 "
        }
        response = requests.get(url, headers=headers)
        logger.info(
            f"{response.status_code} {response.reason} | GET {url}  ({humanize.naturalsize(len(response.content))} Bytes)"
        )
        return response


class BGPViewCIDR(CIDR):
    def __init__(self, query_term_list: List[str] = []):
        self.query_term_list = query_term_list
        self.bgpview_client = bgpview.BGPViewClient()

    def ipv4_prefixes(self) -> List[str]:
        ipv4_networks = []
        for query_term in self.query_term_list:
            data = self.bgpview_client.search(query_term)
            for item in data["data"]["ipv4_prefixes"]:
                ipv4_networks.append(str(ipaddress.IPv4Network(item["prefix"], strict=False)))
        return deduplicate_networks(ipv4_networks)

    def ipv6_prefixes(self) -> List[str]:
        ipv6_networks = []
        for query_term in self.query_term_list:
            data = self.bgpview_client.search(query_term)
            for item in data["data"]["ipv6_prefixes"]:
                ipv6_networks.append(str(ipaddress.IPv6Network(item["prefix"], strict=False)))
        return deduplicate_networks(ipv6_networks)


@dataclass
class CNAMEPattern:
    suffix: str = ""
    pattern: str = ""
    source: str = ""

    def __str__(self) -> str:
        return self.suffix

    def __repr__(self) -> str:
        return self.suffix

    def marshal(self) -> dict:
        return {
            "suffix": self.suffix,
            "pattern": self.pattern,
            "source": self.source,
            "subscribers": self.subscribers(),
        }

    def subscribers(self) -> List[str]:
        if not os.getenv("MONGODB_URI"):
            logger.warning("MONGODB_URI is not set")
            return []

        suffix = self.suffix if self.suffix.endswith(".") else self.suffix + "."
        pattern = f'^{suffix[::-1].replace(".", chr(92)+".")}'
        filter = {
            "task.qtype": "A",
            "task.dns.response.answers.cname_reverse": {"$regex": pattern},
            "task.dns.response.answers.a": {"$exists": True, "$ne": []},
        }
        collection = db.get_mongo_collection("kepler", "dns")
        domain_set = set()
        for item in collection.find(filter).limit(1):
            qname = item["task"]["qname"]
            logger.warning("{} | {}", qname, self.suffix)
            domain_set.add(qname)
        return list(domain_set)


@dataclass
class CommonCDN:
    name: str
    asn_patterns: list[str] = field(default_factory=list)
    cname_suffixes: list[CNAMEPattern] = field(default_factory=list)

    def __init__(
        self,
        name: str,
        asn_patterns: List[str],
        cname_suffixes: List[CNAMEPattern],
        cidr: CIDR,
    ):
        self.name = name
        self.asn_patterns = asn_patterns
        self.cname_suffixes = cname_suffixes
        self.cidr = cidr

    def ipv4_prefixes(self) -> List[str]:
        return self.cidr.ipv4_prefixes()

    def ipv6_prefixes(self) -> List[str]:
        return self.cidr.ipv6_prefixes()

    def subscribers(self) -> List[str]:
        results = set()
        for cname_suffix in self.cname_suffixes:
            for subscriber in cname_suffix.subscribers():
                results.add(subscriber)
        return list(results)

    def __str__(self) -> str:
        return self.name

    def __repr__(self) -> str:
        return self.name

    def dump(self) -> bool:
        path = os.path.join(
            os.path.dirname(
                os.path.dirname(
                    os.path.dirname(
                        os.path.dirname(__file__),
                    ),
                ),
            ),
            "assets",
            "cdn",
            f"{self.name}.yaml",
        )
        with open(path, mode="r", encoding="utf-8") as f:
            old_content = f.read()

        with tempfile.NamedTemporaryFile(mode="w", encoding="utf-8", delete=False) as f:
            temporary_filename = f.name
            yaml.dump(
                {
                    "name": self.name,
                    "asn_patterns": self.asn_patterns,
                    "cname_suffixes": [i.marshal() for i in self.cname_suffixes],
                    "ipv4_prefixes": self.ipv4_prefixes(),
                    "ipv6_prefixes": self.ipv6_prefixes(),
                    "updated_at": datetime.datetime.now().isoformat(),
                },
                f,
            )

        with open(temporary_filename, mode="r", encoding="utf-8") as f:
            new_content = f.read()

        old_lines = [line for line in old_content.splitlines() if not line.startswith("updated_at:")]
        new_lines = [line for line in new_content.splitlines() if not line.startswith("updated_at:")]

        if old_lines != new_lines:
            # move temporary file to the original file
            os.replace(temporary_filename, path)
            return True
        else:
            # remove temporary file
            os.remove(temporary_filename)
            return False


__all__ = [
    "advancedhosting",
    "akamai",
    "alibaba_cdn",
    "alibaba_dcdn",
    "alibaba_waf",
    "asia_isp",
    "baidu",
    "baishan",
    "beluga",
    "bunny",
    "cachefly",
    "cdn77",
    "cdnetworks",
    "cdnvideo",
    "chinacache",
    "cloudflare",
    "cloudfront",
    "ctyun",
    "dnion",
    "douyin",
    "edgecast",
    "edgenext",
    "edgio",
    "fastly_edge_compute",
    "fastly",
    "frontdoor",
    "gcore",
    "google",
    "huawei",
    "imperva",
    "incapsula",
    "jingdong",
    "keycdn",
    "kingsoft",
    "leaseweb",
    "lumen",
    "maxcdn",
    "medianova",
    "qiniu",
    "tata_communications",
    "tencent",
    "ucloud",
    "upyun",
    "verizon",
    "volc",
    "wangsu",
]



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="dnion",
    asn_patterns=["dnion"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(["dnion"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cdn77",
    asn_patterns=["cdn77"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(["cdn77"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="medianova",
    asn_patterns=["medianova"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(query_term_list=["medianova"]),
)



import IPy


def deduplicate_networks(networks, filter_version=4):
    deduplicated_networks = IPy.IPSet(
        [
            IPy.IP(str(network), make_net=1)
            for network in networks
            if IPy.IP(str(network), make_net=1).version() == filter_version
        ]
    )
    deduplicated_networks.optimize()
    return [str(network) for network in deduplicated_networks]



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="huawei",
    asn_patterns=["huawei", "futurewei"],
    cname_suffixes=[
        CNAMEPattern(suffix=".cdnhwc1.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc1.cn"),
        CNAMEPattern(suffix=".cdnhwc1.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc1.com"),
        CNAMEPattern(suffix=".cdnhwc2.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc2.cn"),
        CNAMEPattern(suffix=".cdnhwc2.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc2.com"),
        CNAMEPattern(suffix=".cdnhwc22.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc22.com"),
        CNAMEPattern(suffix=".cdnhwc3.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc3.cn"),
        CNAMEPattern(suffix=".cdnhwc3.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc3.com"),
        CNAMEPattern(suffix=".cdnhwc4.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc4.cn"),
        CNAMEPattern(suffix=".cdnhwc4.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc4.com"),
        CNAMEPattern(suffix=".cdnhwc5.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc5.cn"),
        CNAMEPattern(suffix=".cdnhwc5.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc5.com"),
        CNAMEPattern(suffix=".cdnhwc6.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc6.cn"),
        CNAMEPattern(suffix=".cdnhwc6.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc6.com"),
        CNAMEPattern(suffix=".cdnhwc7.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc7.cn"),
        CNAMEPattern(suffix=".cdnhwc7.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc7.com"),
        CNAMEPattern(suffix=".cdnhwc8.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc8.cn"),
        CNAMEPattern(suffix=".cdnhwc8.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc8.com"),
        CNAMEPattern(suffix=".cdnhwc9.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc9.cn"),
        CNAMEPattern(suffix=".cdnhwc9.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc9.com"),
        CNAMEPattern(suffix=".cdnhwc10.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc10.cn"),
        CNAMEPattern(suffix=".cdnhwc10.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwc10.com"),
        CNAMEPattern(suffix=".cdnhwcaip122.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcaip122.cn"),
        CNAMEPattern(suffix=".cdnhwcajk17.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcajk17.com"),
        CNAMEPattern(suffix=".cdnhwcaoc115.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcaoc115.cn"),
        CNAMEPattern(suffix=".cdnhwcatq08.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcatq08.com"),
        CNAMEPattern(suffix=".cdnhwcbie120.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcbie120.cn"),
        CNAMEPattern(suffix=".cdnhwcbni108.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcbni108.com"),
        CNAMEPattern(suffix=".cdnhwcbqs106.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcbqs106.com"),
        CNAMEPattern(suffix=".cdnhwcbvo20.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcbvo20.cn"),
        CNAMEPattern(suffix=".cdnhwcbzj102.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcbzj102.com"),
        CNAMEPattern(suffix=".cdnhwcchh18.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcchh18.com"),
        CNAMEPattern(suffix=".cdnhwccmz121.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwccmz121.com"),
        CNAMEPattern(suffix=".cdnhwccvo19.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwccvo19.cn"),
        CNAMEPattern(suffix=".cdnhwcdkd21.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcdkd21.cn"),
        CNAMEPattern(suffix=".cdnhwcdvg22.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcdvg22.cn"),
        CNAMEPattern(suffix=".cdnhwcead111.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcead111.com"),
        CNAMEPattern(suffix=".cdnhwcedi10.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcedi10.com"),
        CNAMEPattern(suffix=".cdnhwcedt124.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcedt124.com"),
        CNAMEPattern(suffix=".cdnhwceft18.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwceft18.cn"),
        CNAMEPattern(suffix=".cdnhwceod109.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwceod109.cn"),
        CNAMEPattern(suffix=".cdnhwcfzp15.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcfzp15.cn"),
        CNAMEPattern(suffix=".cdnhwcggk22.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcggk22.com"),
        CNAMEPattern(suffix=".cdnhwcgnc118.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcgnc118.com"),
        CNAMEPattern(suffix=".cdnhwcgqa21.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcgqa21.com"),
        CNAMEPattern(suffix=".cdnhwcgqs21.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcgqs21.com"),
        CNAMEPattern(suffix=".cdnhwcgrb01.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcgrb01.cn"),
        CNAMEPattern(suffix=".cdnhwcgvs16.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcgvs16.cn"),
        CNAMEPattern(suffix=".cdnhwchcg02.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwchcg02.com"),
        CNAMEPattern(suffix=".cdnhwcibv122.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcibv122.com"),
        CNAMEPattern(suffix=".cdnhwcick110.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcick110.com"),
        CNAMEPattern(suffix=".cdnhwcidu117.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcidu117.cn"),
        CNAMEPattern(suffix=".cdnhwcjbj05.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcjbj05.cn"),
        CNAMEPattern(suffix=".cdnhwcjkl10.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcjkl10.cn"),
        CNAMEPattern(suffix=".cdnhwcjlg112.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcjlg112.com"),
        CNAMEPattern(suffix=".cdnhwcjog12.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcjog12.com"),
        CNAMEPattern(suffix=".cdnhwcjqv17.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcjqv17.cn"),
        CNAMEPattern(suffix=".cdnhwcjsb120.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcjsb120.com"),
        CNAMEPattern(suffix=".cdnhwckbw08.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwckbw08.cn"),
        CNAMEPattern(suffix=".cdnhwckfd24.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwckfd24.cn"),
        CNAMEPattern(suffix=".cdnhwckfz116.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwckfz116.com"),
        CNAMEPattern(suffix=".cdnhwckon103.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwckon103.com"),
        CNAMEPattern(suffix=".cdnhwcljk104.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcljk104.com"),
        CNAMEPattern(suffix=".cdnhwclkl04.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwclkl04.cn"),
        CNAMEPattern(suffix=".cdnhwcllh11.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcllh11.com"),
        CNAMEPattern(suffix=".cdnhwclnh23.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwclnh23.cn"),
        CNAMEPattern(suffix=".cdnhwclrd102.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwclrd102.cn"),
        CNAMEPattern(suffix=".cdnhwclsl119.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwclsl119.cn"),
        CNAMEPattern(suffix=".cdnhwclxu105.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwclxu105.com"),
        CNAMEPattern(suffix=".cdnhwclxw05.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwclxw05.com"),
        CNAMEPattern(suffix=".cdnhwcoem01.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcoem01.com"),
        CNAMEPattern(suffix=".cdnhwcofl101.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcofl101.cn"),
        CNAMEPattern(suffix=".cdnhwcogx108.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcogx108.cn"),
        CNAMEPattern(suffix=".cdnhwcohm19.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcohm19.com"),
        CNAMEPattern(suffix=".cdnhwcojn124.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcojn124.cn"),
        CNAMEPattern(suffix=".cdnhwconz06.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwconz06.cn"),
        CNAMEPattern(suffix=".cdnhwcoph123.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcoph123.com"),
        CNAMEPattern(suffix=".cdnhwcozb112.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcozb112.cn"),
        CNAMEPattern(suffix=".cdnhwcpcw110.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcpcw110.cn"),
        CNAMEPattern(suffix=".cdnhwcpqh13.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcpqh13.cn"),
        CNAMEPattern(suffix=".cdnhwcprh113.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcprh113.com"),
        CNAMEPattern(suffix=".cdnhwcpsd13.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcpsd13.com"),
        CNAMEPattern(suffix=".cdnhwcqgw115.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcqgw115.com"),
        CNAMEPattern(suffix=".cdnhwcqir15.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcqir15.com"),
        CNAMEPattern(suffix=".cdnhwcqrr114.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcqrr114.cn"),
        CNAMEPattern(suffix=".cdnhwcqve117.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcqve117.com"),
        CNAMEPattern(suffix=".cdnhwcqvk105.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcqvk105.cn"),
        CNAMEPattern(suffix=".cdnhwcqwg14.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcqwg14.com"),
        CNAMEPattern(suffix=".cdnhwcrgg07.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcrgg07.cn"),
        CNAMEPattern(suffix=".cdnhwcrpp12.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcrpp12.cn"),
        CNAMEPattern(suffix=".cdnhwcscc123.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcscc123.cn"),
        CNAMEPattern(suffix=".cdnhwcsjs09.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcsjs09.cn"),
        CNAMEPattern(suffix=".cdnhwcsux118.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcsux118.cn"),
        CNAMEPattern(suffix=".cdnhwctmz11.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwctmz11.cn"),
        CNAMEPattern(suffix=".cdnhwctnm107.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwctnm107.com"),
        CNAMEPattern(suffix=".cdnhwctxz24.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwctxz24.com"),
        CNAMEPattern(suffix=".cdnhwctyd116.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwctyd116.cn"),
        CNAMEPattern(suffix=".cdnhwcuim119.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcuim119.com"),
        CNAMEPattern(suffix=".cdnhwcupf06.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcupf06.com"),
        CNAMEPattern(suffix=".cdnhwcurq03.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcurq03.com"),
        CNAMEPattern(suffix=".cdnhwcurr103.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcurr103.cn"),
        CNAMEPattern(suffix=".cdnhwcuwd107.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcuwd107.cn"),
        CNAMEPattern(suffix=".cdnhwcuxc121.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcuxc121.cn"),
        CNAMEPattern(suffix=".cdnhwcvix16.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcvix16.com"),
        CNAMEPattern(suffix=".cdnhwcwqz113.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcwqz113.cn"),
        CNAMEPattern(suffix=".cdnhwcxcy07.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcxcy07.com"),
        CNAMEPattern(suffix=".cdnhwcxsi14.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcxsi14.cn"),
        CNAMEPattern(suffix=".cdnhwcywp03.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcywp03.cn"),
        CNAMEPattern(suffix=".cdnhwcyxb02.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcyxb02.cn"),
        CNAMEPattern(suffix=".cdnhwczba04.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczba04.com"),
        CNAMEPattern(suffix=".cdnhwczic111.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczic111.cn"),
        CNAMEPattern(suffix=".cdnhwczid104.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczid104.cn"),
        CNAMEPattern(suffix=".cdnhwczjt20.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczjt20.com"),
        CNAMEPattern(suffix=".cdnhwczks109.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczks109.com"),
        CNAMEPattern(suffix=".cdnhwczmn114.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczmn114.com"),
        CNAMEPattern(suffix=".cdnhwczoy106.cn", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczoy106.cn"),
        CNAMEPattern(suffix=".cdnhwczth23.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczth23.com"),
        CNAMEPattern(suffix=".cdnhwcztu09.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwcztu09.com"),
        CNAMEPattern(suffix=".cdnhwczxh101.com", pattern=r"${domain}.[0-9a-f]{8}.cdnhwczxh101.com"),
    ],
    cidr=BGPViewCIDR(["huawei", "futurewei"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="cdnvideo",
    asn_patterns=["cdnvideo"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(["cdnvideo"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="qiniu",
    asn_patterns=["qiniu"],
    cname_suffixes=[
        CNAMEPattern(suffix=".qiniudns.com", pattern=r"${dot2dash-domain}-id[0-9a-z]{5}.qiniudns.com"),
    ],
    cidr=BGPViewCIDR(query_term_list=["qiniu"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="keycdn",
    asn_patterns=["keycdn"],
    cname_suffixes=[
        CNAMEPattern(suffix=".kxcdn.com"),
    ],
    cidr=BGPViewCIDR(["keycdn"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="alibaba-waf",
    asn_patterns=["alibaba", "taobao", "alicloud"],
    cname_suffixes=[
        CNAMEPattern(suffix=".yundunwaf1.com", pattern=r"${random}.yundunwaf1.com"),
        CNAMEPattern(suffix=".yundunwaf2.com", pattern=r"${random}.yundunwaf2.com"),
        CNAMEPattern(suffix=".yundunwaf3.com", pattern=r"${random}.yundunwaf3.com"),
        CNAMEPattern(suffix=".yundunwaf4.com", pattern=r"${random}.yundunwaf4.com"),
        CNAMEPattern(suffix=".yundunwaf5.com", pattern=r"${random}.yundunwaf5.com"),
    ],
    cidr=BGPViewCIDR(["alibaba", "taobao", "alicloud"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="baishan",
    asn_patterns=["baishan"],
    cname_suffixes=[
        CNAMEPattern(suffix=".bsgslb.cn", pattern=r"${domain}.bsgslb.cn"),
        CNAMEPattern(suffix=".trpcdn.net", pattern=r"${domain}.bsgslb.cn"),
    ],
    cidr=BGPViewCIDR(["baishan"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="tata-communications",
    asn_patterns=["tata"],
    cname_suffixes=[],
    cidr=BGPViewCIDR(query_term_list=["tata"]),
)



from cdnmon.model.cdn import BGPViewCIDR
from cdnmon.model.cdn import CNAMEPattern
from cdnmon.model.cdn import CommonCDN

CDN = CommonCDN(
    name="kingsoft",
    asn_patterns=["kingsoft", "ksyun"],
    cname_suffixes=[
        CNAMEPattern(suffix=".download.ks-cdn.com", pattern=r"${domain}.download.ks-cdn.com"),
        CNAMEPattern(suffix=".ksyuncdn.com"),
    ],
    cidr=BGPViewCIDR(["kingsoft", "ksyun"]),
)
